import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Event } from "@mui/icons-material";
import { ClickAwayListener, TextField, Box, useTheme, useMediaQuery, IconButton, } from "@mui/material";
import { LocalizationProvider, StaticDatePicker, } from "@mui/x-date-pickers-pro";
import { DateTime } from "luxon";
import { createRef, useEffect, useRef, useState, } from "react";
import { useTranslation } from "react-i18next";
import i18n from "../i18n";
import { NerdPickerWrapper } from "../nerdPickersWrapper";
import { CustomLuxonAdapter } from "./customLuxonAdapter";
import { FormControl, FormHelperText, FormLabel, Input, } from "@mui/joy";
const defaultlocaleStringFormat = {
    weekday: "long",
    month: "long",
    day: "2-digit",
};
/**
 *
 * @param {NerdDatePickerProps} props Props
 * @returns {ReactElement} NerdDateTimePicker
 */
export default function NerdDatePicker(props) {
    const { setOpenCalendar, openCalendar, inputProps, helperText, label, datePickerProps, onChange, value, localeStringFormat = defaultlocaleStringFormat, formLabelProps, } = props;
    const [textFieldValue, setTextFieldValue] = useState(value !== null ? value.toLocaleString(localeStringFormat) : "");
    const [open, setOpen] = useState({
        anchorEl: null,
        triggeredBy: null,
    });
    const [tmpValue, setTmpValue] = useState(value);
    const adapter = new CustomLuxonAdapter({ locale: i18n.language });
    const theme = useTheme();
    const isDownMd = useMediaQuery(theme.breakpoints.down("md"));
    const { t } = useTranslation("components", { i18n: i18n });
    const inputRef = createRef();
    const iconButtonRef = createRef();
    const [disabledDate, setDisabledDate] = useState(false);
    const popperRef = useRef(null);
    const _helperText = ((textFieldValue && !adapter.date(textFieldValue)?.isValid) ||
        disabledDate) &&
        !open.anchorEl
        ? t("nerdDatePicker.invalid_date")
        : helperText ?? "";
    const handleClick = (triggeredBy) => {
        if (!open.anchorEl) {
            setOpen({ anchorEl: iconButtonRef.current, triggeredBy });
            setOpenCalendar?.(true);
        }
        else
            handleClose();
    };
    const handleClose = () => {
        if (open.anchorEl !== null) {
            setOpen({ anchorEl: null, triggeredBy: null });
            setOpenCalendar?.(false);
        }
    };
    useEffect(() => {
        setTextFieldValue(value?.toLocaleString(localeStringFormat) || "");
        setTmpValue(value);
    }, [value]);
    useEffect(() => {
        if (openCalendar === true)
            setOpen({ anchorEl: iconButtonRef.current, triggeredBy: "input" });
        else
            handleClose();
    }, [openCalendar]);
    const onTextFieldValueChange = (newValue) => {
        setDisabledDate(false);
        setTextFieldValue(newValue);
    };
    const onTextFieldBlur = (e) => {
        if (popperRef.current?.contains(e.relatedTarget) ||
            e.relatedTarget?.classList.contains("MuiPickersDay-root") ||
            e.relatedTarget?.classList.contains("DatePickerPopper")) {
            inputRef.current?.focus();
        }
        else {
            const date = adapter.date(e.target.value);
            if (date?.isValid) {
                const newDate = value?.set({
                    day: date.day,
                    month: date.month,
                    ...(e.target.value.includes(date.year.toString()) && {
                        year: date.year,
                    }),
                });
                if (newDate?.toMillis() != value?.toMillis()) {
                    if (newDate && datePickerProps?.shouldDisableDate?.(newDate)) {
                        setDisabledDate(true);
                    }
                    else {
                        setTextFieldValue(date.toLocaleString(localeStringFormat));
                        onChange(newDate || null);
                    }
                }
                else
                    setTextFieldValue(value?.toLocaleString(localeStringFormat) || "");
            }
            handleClose();
        }
    };
    const onDateChange = (newValue) => {
        const date = adapter.date(newValue);
        if (date) {
            setTmpValue(date.isValid ? date : null);
            setTextFieldValue(date.toLocaleString(localeStringFormat));
            const newDate = value?.set({
                day: date.day,
                month: date.month,
                year: date.year,
            }) || date;
            if (newDate?.toMillis() !== value?.toMillis() && !isDownMd) {
                onChange(newDate || null);
            }
        }
    };
    const getActions = () => {
        const minDate = datePickerProps?.minDate;
        const today = DateTime.now();
        if (minDate) {
            if (typeof minDate !== "string") {
                return minDate.startOf("day") > today.startOf("day") ? [] : ["today"];
            }
            else {
                return DateTime.fromISO(minDate).startOf("day") > today.startOf("day")
                    ? []
                    : ["today"];
            }
        }
        else
            return ["today"];
    };
    return (_jsx(ClickAwayListener, { onClickAway: handleClose, children: _jsx("div", { children: _jsxs(LocalizationProvider, { dateAdapter: CustomLuxonAdapter, adapterLocale: i18n.language, children: [_jsxs(FormControl, { children: [label !== "" ? (_jsx(FormLabel, { ...formLabelProps, children: label ?? t("nerdDatePicker.date") })) : null, _jsx(Input, { fullWidth: true, size: "sm", ...inputProps, slotProps: {
                                    input: {
                                        ref: inputRef,
                                    },
                                }, autoComplete: "off", value: textFieldValue, endDecorator: _jsx(IconButton, { disabled: inputProps?.disabled, ref: iconButtonRef, size: "small", onClick: (e) => {
                                        if (!inputProps?.disabled) {
                                            e.preventDefault();
                                            e.stopPropagation();
                                            handleClick("icon");
                                        }
                                    }, onKeyDown: (e) => {
                                        if (e.code == "Enter" && !inputProps?.disabled) {
                                            e.stopPropagation();
                                            e.preventDefault();
                                            handleClick("icon");
                                        }
                                    }, children: _jsx(Event, {}) }), onClick: (e) => {
                                    if (!inputProps?.disabled)
                                        handleClick("input");
                                    inputProps?.onClick?.(e);
                                }, onChange: (e) => {
                                    onTextFieldValueChange(e.target.value);
                                    inputProps?.onChange?.(e);
                                }, onBlur: (e) => {
                                    onTextFieldBlur(e);
                                    inputProps?.onBlur?.(e);
                                }, error: (textFieldValue && !adapter.date(textFieldValue)?.isValid) ||
                                    disabledDate ||
                                    inputProps?.error
                                    ? true
                                    : false }), _helperText ? (_jsx(FormHelperText, { children: _helperText })) : null] }), _jsx(NerdPickerWrapper, { popperProps: {
                            ref: popperRef,
                            tabIndex: 0,
                            className: "DatePickerPopper",
                            open: open.anchorEl === null ? false : true,
                            anchorEl: open.anchorEl,
                            sx: {
                                zIndex: theme.zIndex.modal,
                                boxShadow: theme.shadows[5],
                                borderRadius: "4px",
                                backgroundColor: "inherit",
                            },
                            onKeyDown: (e) => {
                                if (e.key === "Escape") {
                                    handleClose();
                                    inputRef.current?.focus();
                                }
                            },
                        }, dialogProps: {
                            open: open.anchorEl === null ? false : true,
                            onClose: handleClose,
                        }, dialogActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange(null);
                                handleClose();
                            },
                            onSetToday() {
                                onChange(DateTime.now());
                                handleClose();
                            },
                            actions: datePickerProps?.componentsProps?.actionBar?.actions || [
                                ...getActions(),
                                "cancel",
                                "accept",
                            ],
                        }, popperActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange(null);
                                handleClose();
                            },
                            onSetToday() {
                                onChange(DateTime.now());
                                handleClose();
                            },
                            actions: datePickerProps?.componentsProps?.actionBar?.actions ||
                                getActions(),
                        }, children: _jsx(Box, { sx: {
                                "& .MuiPickerStaticWrapper-content": {
                                    borderRadius: "4px",
                                },
                            }, children: _jsx(StaticDatePicker, { displayStaticWrapperAs: isDownMd ? "mobile" : "desktop", showToolbar: false, componentsProps: {
                                    actionBar: {
                                        actions: [],
                                    },
                                }, disableMaskedInput: true, autoFocus: open.triggeredBy === "icon" ? true : false, value: tmpValue, ...datePickerProps, onChange: (value, keyboardInputValue) => {
                                    onDateChange(value);
                                    if (datePickerProps?.closeOnSelect !== false && !isDownMd) {
                                        handleClose();
                                        inputRef.current?.focus();
                                    }
                                    datePickerProps?.onChange?.(value, keyboardInputValue);
                                }, renderInput: (params) => _jsx(TextField, { ...params }) }) }) })] }) }) }));
}
