import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Button, Dialog, DialogContent, DialogTitle, Divider, Typography, } from "@mui/material";
import { Box } from "@mui/system";
import { useEffect, useState } from "react";
/**
 *
 * @returns {ReactElement} This component ensure that the frontend is up-to-date.
 * it query the endpoint api.versionCheckerEndpoint (in env.js) every X seconds and compares the tags.
 * If they are not the same, a message is displayed.
 */
export default function NerdVersionChecker({ versionCheckerEndpoint, checkRate = 10000, }) {
    const [open, setOpen] = useState(false);
    const [skipped, setSkipped] = useState(false);
    const [newVersion, setNewVersion] = useState("0.0.0");
    let refreshInterval;
    const tag = window.app_info?.tag;
    useEffect(() => {
        if (!refreshInterval && versionCheckerEndpoint) {
            refreshInterval = setInterval(checkNow, checkRate);
            checkNow();
        }
        return clear;
    }, []);
    const clear = () => {
        if (refreshInterval)
            clearInterval(refreshInterval);
        refreshInterval = null;
    };
    const checkNow = () => {
        fetch(versionCheckerEndpoint)
            .then((response) => response.json())
            .then((data) => {
            if (tag !== "alpha" &&
                tag !== "_COMMIT_REF_NAME" &&
                tag !== data.currentVersion &&
                !skipped) {
                setOpen(true);
                setNewVersion(data.currentVersion ?? "");
            }
        });
    };
    return (_jsx(_Fragment, { children: !skipped ? (_jsxs(Dialog, { open: open, onClose: () => {
                setOpen(false);
                setSkipped(true);
            }, children: [_jsxs(DialogTitle, { sx: { textAlign: "center" }, children: [_jsx("i", { style: { fontSize: 48 }, className: "fa-regular fa-circle-down" }), _jsx("br", {}), _jsxs("b", { children: ["v", newVersion] }), ": A new update is available"] }), _jsxs(DialogContent, { sx: {
                        textAlign: "center",
                        gap: 1,
                        display: "flex",
                        flexDirection: "column",
                    }, children: [_jsxs(Box, { children: ["You are currently using an outdated version of the app.", _jsx("br", {}), "Refresh the page to get the latest features and bug fixes."] }), _jsx(Divider, {}), _jsx(Box, { sx: { lineHeight: 0.4 }, children: _jsxs(Typography, { variant: "caption", children: ["If you are currently filling a form, you may loose your content.", _jsx("br", {}), "In this case you can temporarly skip this update and finish what you are currently doing.", _jsx("br", {}), "We encourage you to refresh the page whenever you can to get the update."] }) })] }), _jsxs(DialogContent, { sx: { justifyContent: "end", gap: 1, display: "flex" }, children: [_jsx(Button, { onClick: () => {
                                setSkipped(true);
                                setOpen(false);
                            }, sx: { textTransform: "none" }, size: "small", variant: "outlined", children: "I'll do it later" }), _jsx(Button, { onClick: () => {
                                window.location.reload();
                            }, sx: { textTransform: "none" }, size: "small", variant: "contained", children: "Refresh Now" })] })] })) : ([]) }));
}
